import bz2
from functools import lru_cache
from pathlib import Path

import geopandas as gpd
import pandas as pd


def add_metadata_columns_to_df(
    df,
    metadata_to_add=["kommunedata", "fylkesdata"],
    lat_column_name="lat",
    lon_column_name="lon",
    how="left",
) -> pd.DataFrame:
    valid_metadata_options = ["kommunedata", "fylkesdata"]
    if not all(x in valid_metadata_options for x in metadata_to_add):
        raise KeyError(
            f"Invalid value specified in columns_to_add. Permitted values are {valid_metadata_options}"
        )

    if "kommunedata" in metadata_to_add:
        df = _add_kommune_metadata_columns_to_df(
            df,
            how=how,
            lat_column_name=lat_column_name,
            lon_column_name=lon_column_name,
        )

    if "fylkesdata" in metadata_to_add:
        df = _add_fylke_metadata_columns_to_df(
            df,
            how=how,
            lat_column_name=lat_column_name,
            lon_column_name=lon_column_name,
        )

    return df


def _add_kommune_metadata_columns_to_df(
    df, how, lat_column_name, lon_column_name
) -> pd.DataFrame:
    """Add columns for which kommune and kommunenummer the coordinates belong to."""
    kommunedata = _load_kommunedata()

    goal = gpd.GeoDataFrame(
        df,
        geometry=gpd.points_from_xy(
            df[lon_column_name], df[lat_column_name], crs="EPSG:4326"
        ),
    )
    return gpd.sjoin(goal, kommunedata, how=how, op="within").drop(
        ["geometry", "index_right"], axis=1
    )


def _add_fylke_metadata_columns_to_df(
    df, how, lat_column_name, lon_column_name
) -> pd.DataFrame:
    fylkesdata = _load_fylkesdata()

    goal = gpd.GeoDataFrame(
        df,
        geometry=gpd.points_from_xy(
            df[lon_column_name], df[lat_column_name], crs="EPSG:4326"
        ),
    )
    return gpd.sjoin(goal, fylkesdata, how=how, op="within").drop(
        ["geometry", "index_right"], axis=1
    )


@lru_cache(maxsize=None)
def _load_kommunedata():
    """Loads data for kommuner and kommunenummer"""
    filepath = (
        Path(__file__).joinpath("../resources/Kommuner-2020-large.json.bz2").resolve()
    )
    with bz2.open(filepath, mode="rt", encoding="utf-8") as f:
        return gpd.read_file(f)


@lru_cache(maxsize=None)
def _load_fylkesdata():
    """Loads data for fylker and fylkesnummer"""
    filepath = (
        Path(__file__).joinpath("../resources/Fylker-2020-large.json.bz2").resolve()
    )
    with bz2.open(filepath, mode="rt", encoding="utf-8") as f:
        return gpd.read_file(f)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hvor']

package_data = \
{'': ['*'], 'hvor': ['resources/*']}

install_requires = \
['Rtree>=0.9.7,<0.10.0',
 'geopandas>=0.9.0,<0.10.0',
 'setuptools>=57.4.0,<58.0.0']

setup_kwargs = {
    'name': 'hvor',
    'version': '0.1.1',
    'description': 'A library for assigning Norwegian metadata to coordinates',
    'long_description': "# Hvor\n\nA library for assigning Norwegian metadata to coordinates. For example, if you\nhave a list of coordinates, and you'd like to know which county and municipality\neach coordinate belongs to, `hvor` can help you.\n\n## Installation\n\nSimply run\n\n```\npip install hvor\n```\n\n## Usage\n\n```python\nfrom hvor import add_metadata_columns_to_df\n\ndf = add_metadata_columns_to_df(df)\n```\n\nVoila! County and municipality metadata for your coordinates have been added to\nyour dataframe (\\*but only if your latitude and longitude columns were called\n`lat` and `lon`😅).\n\n## Credits\n\nBig thanks to [robhop](https://github.com/robhop) for sharing his lightly\nprocessed county and municipality polygons. Also big thanks to Kartverket, for\nsupplying the original dataset.\n",
    'author': 'Espen Hafstad Solvang',
    'author_email': 'espenhafstadsolvang@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bkkas/iout_foss',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
